//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Interface Demonstration (AdvancePrompt & AdvanceMath)
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  Carnegie Mellon University
//                  5000 Forbes Avenue
//                  Pittsburgh, PA 15213-3891
//                  e-mail: pattis@cs.cmu.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This program demonstrates a simple but powerful use of interfaces.
// There are two salient features about interfaces.
//
//   First, a class that implements an interface (it says so, right in the
// class definition) must define all the methods mentioned in the
// interface, using those exact protottypes (it can also define
// constructors and other methods too). The interface specifies the
// minimum definition.
//
//   Second, we can use the interface name as a parameter type in a
// method. We can pass to that parameter an object that is constructted
// from any class that implements that interface. What can we do with
// such objects: call any of the methods specified in the interface,
// because all objects passed to the parameter can execute those methods.
// Thus, we now have an interface type (for the parameter) that can refer
// to objects of many types (with the connection that each refered-to type
// implements the interface).
//
//   Here is what we will decmonstrate (two library classes that define
// an interesting static method). 
//
//    1. An AdvancePrompt class defines a forInt method that takes an
//       object from a class implementing DecisionInt, to filter good
//       from bad inputs. The user enters a value until it is deemed
//       OK according to the passed object.
//
//    2. An AdvanceMath class defines an integrate method that takes an
//       object from a class implementing Univariate, to act as the
//       function to integrate, between a lower and upper bound (wiht
//       a given step).
//
//   Finally, because of the large number of classes used, we will start
// to group classes in subfolders.
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   9/11/01: R. Pattis - Operational in Java
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import edu.cmu.cs.pattis.cs151xx.Prompt;
import edu.cmu.cs.pattis.cs151xx.AdvancedPrompt;
import edu.cmu.cs.pattis.cs151xx.IsEven;
import edu.cmu.cs.pattis.cs151xx.IsPositive;
import edu.cmu.cs.pattis.cs151xx.IsPrime;
import edu.cmu.cs.pattis.cs151xx.IsBetween;

import edu.cmu.cs.pattis.cs151xx.Linear;
import edu.cmu.cs.pattis.cs151xx.SpecialForm;
import edu.cmu.cs.pattis.cs151xx.AdvancedMath;



public class Application {

	public static void main(String[] args)
	{
    //Test various DecisionInt filters	  
	  
	  int even = AdvancedPrompt.forInt("\nEnter an even",new IsEven(),"Error: evens are divisible by 2");
	  System.out.println("even = " + 2);
		  
	  int positive = AdvancedPrompt.forInt("\nEnter a positive",new IsPositive(),"Error: positives are > 0");
	  System.out.println("positive = " + positive);
	  
	  int prime = AdvancedPrompt.forInt("\nEnter a prime",new IsPrime(),"Error: primes are >= 2, divisible by only 1 and themselves");
	  System.out.println("prime = " + prime);

	  int between = AdvancedPrompt.forInt("\nEnter in [0,10]",new IsBetween(0,10),"Error: must be in range [0,10] inclusive");
	  System.out.println("between = " + between);
	  
	  
	  
	  //Test various Univarate integrals
	  
	  System.out.println();
	  double low1  = Prompt.forDouble("Enter low ");
	  double high1 = Prompt.forDouble("Enter high");
	  double step1 = Prompt.forDouble("Enter step");
	  double a1    = Prompt.forDouble("Enter a (in ax+b)");
	  double b1    = Prompt.forDouble("Enter b (in ax+b)");
	  System.out.println("Integrate " + a1+"x+"+b1+ "(from "+low1+" to "+high1+")= " + 
	                     AdvancedMath.integrate(new Linear(a1,b1), low1, high1, step1));
		  
	  System.out.println();
	  double low2  = Prompt.forDouble("Enter low ");
	  double high2 = Prompt.forDouble("Enter high");
	  double step2 = Prompt.forDouble("Enter step");
	  double a2    = Prompt.forDouble("Enter a (in a/math.Sqrt(b*x*x+c*x))");
	  double b2    = Prompt.forDouble("Enter b (in a/math.Sqrt(b*x*x+c*x))");
	  double c2    = Prompt.forDouble("Enter c (in a/math.Sqrt(b*x*x+c*x))");
	  System.out.println("Integrate " + a2+"/Math.sqrt("+b2+ "*x*x + "+c2+"*x)  (from "+low2+" to "+high2+")= " + 
	                     AdvancedMath.integrate(new SpecialForm(a2,b2,c2), low2, high2, step2));
	}
}
